
#include <config.h>

#include "rcsid.h"
RCSID ("$Id: shadowio.c,v 1.15 2005/05/25 18:20:22 kloczek Exp $")
#include "prototypes.h"
#include "defines.h"
#ifdef HAVE_SHADOW_H
# include <shadow.h>
#endif
#ifdef SHADOWTCB
# include <tcb.h>
# include "tcbfuncs.h"
#endif
#include <stdio.h>
#include "commonio.h"
#include "shadowio.h"
#include "getdef.h"

struct spwd *__spw_dup (const struct spwd *spent)
{
	struct spwd *sp;

	if (!(sp = (struct spwd *) malloc (sizeof *sp)))
		return NULL;
	*sp = *spent;
	if (!(sp->sp_namp = strdup (spent->sp_namp)))
		return NULL;
	if (!(sp->sp_pwdp = strdup (spent->sp_pwdp)))
		return NULL;
	return sp;
}

static void *shadow_dup (const void *ent)
{
	const struct spwd *sp = ent;

	return __spw_dup (sp);
}

static void shadow_free (void *ent)
{
	struct spwd *sp = ent;

	free (sp->sp_namp);
	free (sp->sp_pwdp);
	free (sp);
}

static const char *shadow_getname (const void *ent)
{
	const struct spwd *sp = ent;

	return sp->sp_namp;
}

static void *shadow_parse (const char *line)
{
	return (void *) sgetspent (line);
}

static int shadow_put (const void *ent, FILE * file)
{
	const struct spwd *sp = ent;

	return (putspent (sp, file) == -1) ? -1 : 0;
}

static struct commonio_ops shadow_ops = {
	shadow_dup,
	shadow_free,
	shadow_getname,
	shadow_parse,
	shadow_put,
	fgets,
	fputs
};

struct commonio_db shadow_db = {
	SHADOW_FILE,		/* filename */
	&shadow_ops,		/* ops */
	NULL,			/* fp */
	NULL,			/* head */
	NULL,			/* tail */
	NULL,			/* cursor */
	0,			/* changed */
	0,			/* isopen */
	0,			/* locked */
	0			/* readonly */
};

#ifdef SHADOWTCB
#define tcb_wrapper(FUNC, PARAM, FLAG) \
{ \
}
#endif

int spw_name (const char *filename)
{
	return commonio_setname (&shadow_db, filename);
}

int spw_file_present (void)
{
#ifdef SHADOWTCB
	if (getdef_bool("USE_TCB"))
		return 1;
#endif
	return commonio_present (&shadow_db);
}

int spw_lock (void)
{
#ifdef SHADOWTCB
	int retval = 0;

	if (!getdef_bool("USE_TCB"))
#endif
		return commonio_lock(&shadow_db);
#ifdef SHADOWTCB
	if (!s_drop_priv()) return 0;

	if (lckpwdf_tcb(shadow_db.filename) == 0) {
		shadow_db.locked = 1;
		retval = 1;
	}

	if (!s_gain_priv()) return 0;

	return retval;
#endif
}

int spw_open (int mode)
{
#ifdef SHADOWTCB
	int retval = 0;
	int use_tcb = getdef_bool("USE_TCB");

	if (use_tcb)
		if (!s_drop_priv() != 0)
			return 0;

	retval = commonio_open(&shadow_db, mode);

	if (use_tcb)
		if (!s_gain_priv() != 0)
			return 0;

	return retval;
#else
	return commonio_open (&shadow_db, mode);
#endif
}

const struct spwd *spw_locate (const char *name)
{
	return commonio_locate (&shadow_db, name);
}

int spw_update (const struct spwd *sp)
{
	return commonio_update (&shadow_db, (const void *) sp);
}

int spw_remove (const char *name)
{
	return commonio_remove (&shadow_db, name);
}

int spw_rewind (void)
{
	return commonio_rewind (&shadow_db);
}

const struct spwd *spw_next (void)
{
	return commonio_next (&shadow_db);
}

int spw_close (void)
{
#ifdef SHADOWTCB
	int retval = 0;
	int use_tcb = getdef_bool("USE_TCB");

	if (use_tcb)
		if (!s_drop_priv() != 0)
			return 0;

	retval = commonio_close(&shadow_db);

	if (use_tcb)
		if (!s_gain_priv() != 0)
			return 0;

	return retval;
#else
	return commonio_close (&shadow_db);
#endif
}

int spw_unlock (void)
{
#ifdef SHADOWTCB
	int retval = 0;

	if (!getdef_bool("USE_TCB"))
#endif
		return commonio_unlock(&shadow_db);
#ifdef SHADOWTCB
	if (!s_drop_priv()) return 0;

	if (ulckpwdf_tcb() == 0) {
		shadow_db.locked = 0;
		retval = 1;
	}

	if (!s_gain_priv()) return 0;

	return retval;
#endif
}

struct commonio_entry *__spw_get_head (void)
{
	return shadow_db.head;
}

void __spw_del_entry (const struct commonio_entry *ent)
{
	commonio_del_entry (&shadow_db, ent);
}

/* Sort with respect to passwd ordering. */
int spw_sort ()
{
	extern struct commonio_db *__pw_get_db ();

	return commonio_sort_wrt (&shadow_db, __pw_get_db ());
}
