/*
 * salt.c - generate a random salt string for crypt()
 */

#define _OW_SOURCE
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <crypt.h>

#include "getdef.h"

#define RANDOM_DEVICE			"/dev/urandom"

static int read_loop(int fd, char *buffer, int count)
{
	int offset, block;

	offset = 0;
	while (count > 0) {
		block = read(fd, &buffer[offset], count);

		if (block < 0) {
			if (errno == EINTR) continue;
			return block;
		}
		if (!block) return offset;

		offset += block;
		count -= block;
	}

	return offset;
}

char *
crypt_make_salt(void)
{
	int fd;
	char entropy[16];
	char *retval;

	fd = open(RANDOM_DEVICE, O_RDONLY);
	if (fd < 0) {
		perror("open: " RANDOM_DEVICE);
		exit(1);
	}

	if (read_loop(fd, entropy, sizeof(entropy)) != sizeof(entropy)) {
		close(fd);
		fprintf(stderr, "Unable to obtain entropy from %s\n",
			RANDOM_DEVICE);
		exit(1);
	}

	close(fd);

	retval = crypt_gensalt(getdef_str("CRYPT_PREFIX") ?: "",
		getdef_num("CRYPT_ROUNDS", 0), entropy, sizeof(entropy));
	memset(entropy, 0, sizeof(entropy));
	if (!retval) {
		fprintf(stderr, "Unable to generate a salt, "
			"check your CRYPT_PREFIX and CRYPT_ROUNDS settings.\n");
		exit(1);
	}

	return retval;
}

#if 0
/*
 * Written by Marek Michalkiewicz <marekm@i17linuxb.ists.pwr.wroc.pl>,
 * public domain.
 */

#include <config.h>

#include "rcsid.h"
RCSID ("$Id: salt.c,v 1.8 2005/05/25 19:31:50 kloczek Exp $")
#include <sys/time.h>
#include <stdlib.h>
#include "prototypes.h"
#include "defines.h"
#include "getdef.h"
/*
 * Generate 8 base64 ASCII characters of random salt.  If MD5_CRYPT_ENAB
 * in /etc/login.defs is "yes", the salt string will be prefixed by "$1$"
 * (magic) and pw_encrypt() will execute the MD5-based FreeBSD-compatible
 * version of crypt() instead of the standard one.
 */
char *crypt_make_salt (void)
{
	struct timeval tv;
	static char result[40];

	result[0] = '\0';
	if (getdef_bool ("MD5_CRYPT_ENAB")) {
		strcpy (result, "$1$");	/* magic for the new MD5 crypt() */
	}

	/*
	 * Generate 8 chars of salt, the old crypt() will use only first 2.
	 */
	gettimeofday (&tv, (struct timezone *) 0);
	strcat (result, l64a (tv.tv_usec));
	strcat (result, l64a (tv.tv_sec + getpid () + clock ()));

	if (strlen (result) > 3 + 8)	/* magic+salt */
		result[11] = '\0';

	return result;
}
#endif
